package cls;

import cls.data.Data;
import cls.domain.Domain;
import cls.logic.ChangeNotifier;
import cls.logic.Logic;
import cls.services.Services;
import cls.view.View;

/**
 * This is the entry point for the application.
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 */
public class ColorLinesSequel {

    private static View view;
    private static ChangeNotifier changeNotifier;
    private static Services services;
    private static Logic logic;
    private static Domain domain;
    private static Data data;

    private static final String COLORLINESSEQUELVERSION = "0.0.0";

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private ColorLinesSequel() {
        //Not meant to be instantiated.
    }

    /**
     *
     * @param args
     */
    public static void main(String[] args) {
        build();
        start();
        System.exit(0);
    }

    /**
     * Creates application layers and connects them.
     */
    private static void build() {
        view = new View();
        changeNotifier = new ChangeNotifier(view);
        data = new Data();
        domain = new Domain(data);
        logic = new Logic(domain, changeNotifier);
        services = new Services(logic);
    }

    /**
     * Starts the View.
     */
    private static void start() {
        //All main game parts have been loaded. View has now the control.
        view.start();
    }

    /**
     *
     * @return
     */
    public static String getColorLinesSequelVersion() {
        return COLORLINESSEQUELVERSION;
    }

    public static View getView() {
        return view;
    }

    public static Logic getLogic() {
        return logic;
    }

    public static Services getServices() {
        return services;
    }
}
