package cls.domain;

import cls.logic.composition.BallDetonatorComposition;
import cls.logic.composition.ExplodingShapeType;
import okay.database.ResultOfSqlQuery;
import okay.json.JsonObject;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class BallDetonatorTable extends Table {

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private BallDetonatorTable() {
        //Not meant to be instantiated.
    }

    static BallDetonatorComposition getBallDetonatorComposition(int ballDetonatorCompositionId) {
        BallDetonatorComposition ballDetonatorComposition;
        JsonObject ballDetonatorCompositionJsonObject = databaseConnection.getRow("ball_detonator", ballDetonatorCompositionId);

        int explodingShapeTypeId = Integer.parseInt(ballDetonatorCompositionJsonObject.getString("exploding_shape_type_id"));

        ExplodingShapeType explodingShapeType = ExplodingShapeTypeTable.getExplodingShapeType(explodingShapeTypeId);

        ballDetonatorComposition = new BallDetonatorComposition(
                explodingShapeType,
                Integer.parseInt(ballDetonatorCompositionJsonObject.getString("minimum_lenght_line")),
                Integer.parseInt(ballDetonatorCompositionJsonObject.getString("minimum_block_size")),
                null
        );
        return ballDetonatorComposition;
    }

    /**
     *
     * @param explodingShapeTypeId
     * @param minimumLineLenght
     * @param minimumBlockSize
     * @param customExplodingShapeId
     * @return
     */
    public static int saveBallDetonator(int explodingShapeTypeId, int minimumLineLenght, int minimumBlockSize, int customExplodingShapeId) {
        StringBuilder insertCommandStringBuilder = new StringBuilder("INSERT INTO ball_detonator VALUES (null,");
        insertCommandStringBuilder.append(explodingShapeTypeId).append(",");
        insertCommandStringBuilder.append(minimumLineLenght).append(",");
        insertCommandStringBuilder.append(minimumBlockSize).append(",");
        insertCommandStringBuilder.append(customExplodingShapeId).append(")");
        return databaseConnection.execute(insertCommandStringBuilder.toString());
    }

    /**
     *
     * @param ballDetonatorComposition
     * @return
     */
    public static int getId(BallDetonatorComposition ballDetonatorComposition) {
        int explodingShapeTypeId = ExplodingShapeTypeTable.getId(ballDetonatorComposition.getExplodingShapeType().toString());
        int minimumLineLenght = ballDetonatorComposition.getMinimumLineLenght();
        int minimumSizeBlock = ballDetonatorComposition.getMinimumBlockSize();

        String sqlQueryString
                = "SELECT * FROM ball_detonator WHERE "
                + "exploding_shape_type_id = "
                + explodingShapeTypeId
                + " AND minimum_lenght_line= "
                + minimumLineLenght
                + " AND minimum_block_size= "
                + minimumSizeBlock;

        ResultOfSqlQuery resultOfSqlQuery = databaseConnection.executeAndReturn(sqlQueryString);
        if (resultOfSqlQuery.isEmpty()) {
            return saveBallDetonator(explodingShapeTypeId, minimumLineLenght, minimumSizeBlock, 0);
        } else {
            resultOfSqlQuery.moveToTheNextRow();
            return resultOfSqlQuery.getInt("id");
        }
    }
}
