package cls.domain;

import cls.logic.composition.BallThrowerComposition;
import okay.database.ResultOfSqlQuery;
import okay.json.JsonObject;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class BallThrowerTable extends Table {

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private BallThrowerTable() {
        //Not meant to be instantiated.
    }

    static BallThrowerComposition getBallThrowerComposition(int ballThrowerCompositionId) {
        BallThrowerComposition ballThrowerComposition;
        JsonObject ballThrowerCompositionJsonObject = databaseConnection.getRow("ball_thrower", ballThrowerCompositionId);

        ballThrowerComposition = new BallThrowerComposition(
                Integer.parseInt(ballThrowerCompositionJsonObject.getString("count_of_balls_thrown_at_the_beginning_of_the_game")),
                Integer.parseInt(ballThrowerCompositionJsonObject.getString("count_of_balls_throwing_during_the_game")),
                Integer.parseInt(ballThrowerCompositionJsonObject.getString("ball_frequency")),
                Integer.parseInt(ballThrowerCompositionJsonObject.getString("automatic_bomb_frequency")),
                Integer.parseInt(ballThrowerCompositionJsonObject.getString("manual_bomb_frequency")),
                Integer.parseInt(ballThrowerCompositionJsonObject.getString("show_the_positions_of_the_next_balls")) == 1
        );
        return ballThrowerComposition;
    }

    /**
     *
     * @param countOfBallsThrownAtTheBeginningOfTheGame
     * @param countOfBallsThrowingDuringTheGame
     * @param ballFrequency
     * @param automaticBombFrequency
     * @param manualBombFrequency
     * @param showThePositionsOfTheNextBalls
     * @return
     */
    public static int saveBallThrower(int countOfBallsThrownAtTheBeginningOfTheGame, int countOfBallsThrowingDuringTheGame, int ballFrequency, int automaticBombFrequency, int manualBombFrequency, boolean showThePositionsOfTheNextBalls) {
        StringBuilder insertCommandStringBuilder = new StringBuilder("INSERT INTO ball_thrower VALUES (null,");
        insertCommandStringBuilder.append(countOfBallsThrownAtTheBeginningOfTheGame).append(",");
        insertCommandStringBuilder.append(countOfBallsThrowingDuringTheGame).append(",");
        insertCommandStringBuilder.append(ballFrequency).append(",");
        insertCommandStringBuilder.append(automaticBombFrequency).append(",");
        insertCommandStringBuilder.append(manualBombFrequency).append(",");
        insertCommandStringBuilder.append(showThePositionsOfTheNextBalls ? 1 : 0).append(")");
        return databaseConnection.execute(insertCommandStringBuilder.toString());
    }

    /**
     *
     * @param ballThrowerComposition
     * @return
     */
    public static int getId(BallThrowerComposition ballThrowerComposition) {
        int countOfBallsThrownAtTheBeginningOfTheGame = ballThrowerComposition.getCountOfBallsThrownAtTheBeginningOfTheGame();
        int countOfBallsThrowingDuringTheGame = ballThrowerComposition.getCountOfBallsThrownDuringTheGame();
        int ballFrequency = ballThrowerComposition.getBallFrequency();
        int automaticBombFrequency = ballThrowerComposition.getAutomaticBombFrequency();
        int manualBombFrequency = ballThrowerComposition.getManualBombFrequency();
        boolean showThePositionsOfTheNextBalls = ballThrowerComposition.getShowingNextBallsPosition();

        String sqlQueryString
                = "SELECT * FROM ball_thrower WHERE "
                + "count_of_balls_thrown_at_the_beginning_of_the_game = "
                + countOfBallsThrownAtTheBeginningOfTheGame
                + " AND count_of_balls_throwing_during_the_game= "
                + countOfBallsThrowingDuringTheGame
                + " AND ball_frequency = "
                + ballFrequency
                + " AND automatic_bomb_frequency = "
                + automaticBombFrequency
                + " AND manual_bomb_frequency = "
                + manualBombFrequency
                + " AND show_the_positions_of_the_next_balls = "
                + (showThePositionsOfTheNextBalls ? "1" : "0");

        ResultOfSqlQuery resultOfSqlQuery = databaseConnection.executeAndReturn(sqlQueryString);
        if (resultOfSqlQuery.isEmpty()) {
            return saveBallThrower(countOfBallsThrownAtTheBeginningOfTheGame, countOfBallsThrowingDuringTheGame, ballFrequency, automaticBombFrequency, manualBombFrequency, showThePositionsOfTheNextBalls);
        } else {
            resultOfSqlQuery.moveToTheNextRow();
            return resultOfSqlQuery.getInt("id");
        }
    }
}
