package cls.domain;

import cls.logic.composition.BoardComposition;
import cls.logic.composition.Shape;
import okay.database.ResultOfSqlQuery;
import okay.json.JsonObject;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class BoardTable extends Table {

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private BoardTable() {
        //Not meant to be instantiated.
    }

    static BoardComposition getBoardComposition(int boardCompositionId) {
        BoardComposition boardComposition;
        JsonObject boardCompositionJsonObject = databaseConnection.getRow("board", boardCompositionId);

        int shapeId = Integer.parseInt(boardCompositionJsonObject.getString("shape_id"));

        Shape shape = ShapeTable.getShape(shapeId);

        boardComposition = new BoardComposition(shape,
                Integer.parseInt(boardCompositionJsonObject.getString("grid_probability")),
                Integer.parseInt(boardCompositionJsonObject.getString("grid_count")),
                Integer.parseInt(boardCompositionJsonObject.getString("wall_probability")),
                Integer.parseInt(boardCompositionJsonObject.getString("wall_count"))
        );
        return boardComposition;
    }

    /**
     *
     * @param gridProbability
     * @param gridCount
     * @param wallProbaility
     * @param wallCount
     * @param shapeId
     * @return
     */
    public static int saveBoard(int gridProbability, int gridCount, int wallProbaility, int wallCount, int shapeId) {
        StringBuilder insertCommandStringBuilder = new StringBuilder("INSERT INTO board VALUES (null,");
        insertCommandStringBuilder.append(gridProbability).append(",");
        insertCommandStringBuilder.append(gridCount).append(",");
        insertCommandStringBuilder.append(wallProbaility).append(",");
        insertCommandStringBuilder.append(wallCount).append(",");
        insertCommandStringBuilder.append(shapeId).append(")");
        return databaseConnection.execute(insertCommandStringBuilder.toString());
    }

    /**
     *
     * @param boardComposition
     * @return
     */
    public static int getId(BoardComposition boardComposition) {
        int gridProbability = boardComposition.getGridProbability();
        int gridCount = boardComposition.getGridCount();
        int wallProbability = boardComposition.getWallProbability();
        int wallCount = boardComposition.getWallCount();

        int shapeId = ShapeTable.getId(boardComposition.getShape());

        String sqlQueryString
                = "SELECT * FROM board,shape WHERE grid_probability = "
                + gridProbability
                + " AND grid_count = "
                + gridCount
                + " AND wall_probability = "
                + wallProbability
                + " AND wall_count = "
                + wallCount
                + " AND shape_id = "
                + shapeId;

        ResultOfSqlQuery resultOfSqlQuery = databaseConnection.executeAndReturn(sqlQueryString);
        if (resultOfSqlQuery.isEmpty()) {
            return saveBoard(gridProbability, gridCount, wallProbability, wallCount, shapeId);
        } else {
            resultOfSqlQuery.moveToTheNextRow();
            return resultOfSqlQuery.getInt("id");
        }
    }
}
