package cls.domain;

import java.util.ArrayList;
import java.util.List;
import okay.database.ResultOfSqlQuery;
import okay.json.JsonObject;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class ColourFrequencyTable extends Table {

    public static final String TABLENAME = "colour_frequency";

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private ColourFrequencyTable() {
        //Not meant to be instantiated.
    }

    /**
     *
     * @param number
     * @return columnName
     */
    public static String getColumnNameForColourFrequency(int number) {
        return "colour" + number + "_frequency";
    }

    static int[] getColourFrequency(int colourFrequencyId) {
        int[] colourFrequency = new int[17];
        JsonObject colourFrequencyJsonObject = databaseConnection.getRow(TABLENAME, colourFrequencyId);

        for (int i = 1; i <= 16; i++) {
            colourFrequency[i] = Integer.parseInt(colourFrequencyJsonObject.getString(getColumnNameForColourFrequency(i)));
        }
        return colourFrequency;
    }

    /**
     *
     * @param colourFrequency
     * @return
     */
    public static int saveColourFrequency(int[] colourFrequency) {
        StringBuilder insertCommandStringBuilder = new StringBuilder("INSERT INTO colour_frequency VALUES (null,");
        for (int i = 1; i <= 16; i++) {
            insertCommandStringBuilder.append(colourFrequency[i]);
            if (i != 16) {
                insertCommandStringBuilder.append(",");
            }
        }
        insertCommandStringBuilder.append(")");
        return databaseConnection.execute(insertCommandStringBuilder.toString());
    }

    /**
     *
     * @param colourFrequency
     * @return
     */
    public static int getId(int[] colourFrequency) {
        List columnList = new ArrayList<>();
        for (int i = 1; i <= 16; i++) {
            columnList.add(getColumnNameForColourFrequency(i));
        }

        StringBuilder stringBuilder = new StringBuilder("SELECT * FROM colour_frequency WHERE ");
        for (int i = 1; i <= 16; i++) {
            stringBuilder.append(getColumnNameForColourFrequency(i)).append(" = ");
            stringBuilder.append(colourFrequency[i]);
            if (i != 16) {
                stringBuilder.append(" AND ");
            }
        }

        ResultOfSqlQuery resultOfSqlQuery = databaseConnection.executeAndReturn(stringBuilder.toString());
        if (resultOfSqlQuery.isEmpty()) {
            return saveColourFrequency(colourFrequency);
        } else {
            resultOfSqlQuery.moveToTheNextRow();
            return resultOfSqlQuery.getInt("id");
        }
    }
}
