package cls.logic;

import cls.ColorLinesSequel;
import cls.domain.ApplicationTable;
import cls.domain.PlayerTable;
import javafx.application.Platform;
import okay.json.JsonObject;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class Application {

    private final Logic logic;
    private boolean running = false;

    private final ApplicationLookForBeforeLogin applicationLookForBeforeLogin = new ApplicationLookForBeforeLogin();

    /**
     * Constructor
     *
     * @param logic
     */
    public Application(Logic logic) {
        this.logic = logic;
    }

    /**
     * Starts logic layer.
     */
    public void start() {
        running = true;
        logic.getLogger().logApplicationStart();

        if (!this.isThereAnAutomaticallyLoggedInPlayer()) {
            logic.addChange("MODE LOGIN");
        } else {
            String nick;
            String password;
            JsonObject player = PlayerTable.getRow(this.getAutomaticallyLoggedInPlayerId());
            nick = player.getString("nick");
            password = player.getString("password");
            logic.getAuthentication().logIn(nick, password, true);
        }
    }

    /**
     * Stops the logic layer.
     */
    public void exit() {
        running = false;
        if (logic.getAuthentication().isLoggedIn()) {
            logic.getAuthentication().logOut();
        }
        logic.getLogger().logApplicationEnd();
        ColorLinesSequel.getLogic().addChange("EXIT");
        Platform.exit();
    }

    /**
     *
     * @return
     */
    public int getAutomaticallyLoggedInPlayerId() {
        return ApplicationTable.getAutomaticallyLoggedInPlayerId();
    }

    public boolean isThereAnAutomaticallyLoggedInPlayer() {
        return this.getAutomaticallyLoggedInPlayerId() != 0;
    }

    public ApplicationLook getApplicationLook() {
        return this.applicationLookForBeforeLogin;
    }

    public boolean isRunning() {
        return running;
    }
}
