package cls.logic;

import cls.logic.game.Game;
import cls.domain.*;
import okay.simplicity.Screen;

/**
 * This class is used to represent all logic in game.
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 */
public class Logic {

    private final ChangeNotifier changeNotifier;

    private final Domain domain;
    private final Application application;

    private ApplicationLook applicationLook;
    private final Authentication authentication;
    private Player player = null;
    private final Logger logger;
    private final GameManager gameManager;
    private final TextSource textSource;

    /**
     *
     * @param domain
     * @param changeNotifier
     */
    public Logic(Domain domain, ChangeNotifier changeNotifier) {
        this.domain = domain;
        this.changeNotifier = changeNotifier;

        this.gameManager = new GameManager(this);

        application = new Application(this);
        authentication = new Authentication(this);
        logger = new Logger();
        this.textSource = new TextSource(this);
        this.applicationLook = this.application.getApplicationLook();
        Screen.setZoom(this.applicationLook.getZoom());
    }

    public Application getApplication() {
        return application;
    }

    public ApplicationLook getApplicationLook() {
        return applicationLook;
    }

    public void setApplicationLook(ApplicationLook applicationLook) {
        this.applicationLook = applicationLook;
    }

    public Authentication getAuthentication() {
        return authentication;
    }

    public Player getPlayer() {
        return player;
    }

    public void setPlayer(Player player) {
        this.player = player;
    }

    public Game getGame() {
        return this.gameManager.getGame();
    }

    public GameManager getGameManager() {
        return this.gameManager;
    }

    public TextSource getTextSource() {
        return this.textSource;
    }

    /**
     * It is used to store a new command in commandBox
     *
     * @param change
     * @return void
     */
    public void addChange(String change) {
        this.changeNotifier.addChange(change);
    }

    Logger getLogger() {
        return this.logger;
    }

}
