package cls.logic.composition;

import cls.ColorLinesSequelRuntimeException;
import cls.logic.composition.lockableobject.LockableObject;
import java.util.ArrayList;
import java.util.List;
import okay.json.JsonArray;
import okay.json.JsonObject;

/**
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 */
public class Shape extends LockableObject {

    private final int height;
    private final int width;
    private final List holeList = new ArrayList<>();

    /**
     *
     * @param shapeJsonObject
     */
    public Shape(JsonObject shapeJsonObject) {
        height = shapeJsonObject.getInt("height");
        width = shapeJsonObject.getInt("width");

        JsonArray holeListJsonArray = shapeJsonObject.getArray("hole list");
        for (int i = 0; i <= holeListJsonArray.getCountOfItems() - 1; i++) {
            Hole hole = new Hole(holeListJsonArray.getObject(i));
            holeList.add(hole);
        }

    }

    /**
     *
     * @param height
     * @param width
     */
    public Shape(int height, int width) {
        if (!((height >= 1) && (width >= 1))) {
            throw new ColorLinesSequelRuntimeException("Height and width must be at least 1.");
        }
        this.height = height;
        this.width = width;
    }

    /**
     * Deletes all holes
     */
    public void deleteAllHoles() {
        this.throwUnsupportedOperationExceptionIfThisObjectIsLocked();
        this.holeList.clear();
    }

    /**
     *
     * @param row
     * @param column
     */
    public void addHole(int row, int column) {
        this.throwUnsupportedOperationExceptionIfThisObjectIsLocked();
        if (row > height) {
            throw new ColorLinesSequelRuntimeException("Row is higher than height");
        }
        if (column > width) {
            throw new ColorLinesSequelRuntimeException("Column is higher than width");
        }
        if (row <= 0) {
            throw new ColorLinesSequelRuntimeException("Column must be at least 1.");
        }
        if (column <= 0) {
            throw new ColorLinesSequelRuntimeException("Column must be at least 1.");
        }
        this.holeList.add(new Hole(column, row));
    }

    /**
     *
     * @return
     */
    public List getHoleListCopy() {
        return new ArrayList<>(this.holeList);
    }

    /**
     *
     */
    @Override
    public void setDefaultValues() {
        this.throwUnsupportedOperationExceptionIfThisObjectIsLocked();
    }

    /**
     *
     */
    @Override
    public void setRandomValues() {
        this.throwUnsupportedOperationExceptionIfThisObjectIsLocked();

    }

    @Override
    public Shape getCopy() {
        Shape shape = new Shape(height, width);
        ArrayList<Hole> list = (ArrayList<Hole>) this.getHoleListCopy();
        for (Hole hole : list) {
            shape.addHole(hole.getRow(), hole.getColumn());
        }
        return shape;
    }

    /**
     *
     * @return
     */
    @Override
    public boolean isThisObjectValid() {
        return true;
    }

    /**
     *
     * @return
     */
    public int getHeight() {
        return this.height;
    }

    /**
     *
     * @return
     */
    public int getWidth() {
        return this.width;
    }

    @Override
    public List getListOfLockableObjects() {
        return new ArrayList<>();
    }

    /**
     *
     * @return
     */
    public JsonObject toJsonObject() {
        JsonObject shapeCompositionJsonObject = new JsonObject();
        shapeCompositionJsonObject.addInt("height", this.getHeight());
        shapeCompositionJsonObject.addInt("width", this.getWidth());

        JsonArray holeListJsonArray = new JsonArray();
        for (Object element : holeList) {
            holeListJsonArray.addObject(((Hole) element).toJsonObject());
        }
        shapeCompositionJsonObject.addArray("hole list", holeListJsonArray);
        return shapeCompositionJsonObject;
    }
}
