package cls.logic.composition.lockableobject;

import java.util.ArrayList;
import java.util.List;
import okay.pseudorandom.PseudoRandomGenerator;

/**
 * Represents an object able to lock to prevent changes. If an object is locked,
 * this object can't be unlocked anymore.
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 */
public abstract class LockableObject {

    private boolean locked = false;

    /**
     * Constructor
     */
    public LockableObject() {
    }

    /**
     * Locks this object.
     */
    public void lockThisObject() {
        this.throwUnsupportedOperationExceptionIfThisObjectIsNotValid();
        lockAllLockableObjectsOfThisObject();
        this.locked = true;
    }

    /**
     * This abstract method is used to execute code after this object was locked
     */
    private void lockAllLockableObjectsOfThisObject() {
        ArrayList<LockableObject> list = (ArrayList<LockableObject>) this.getListOfLockableObjects();
        for (LockableObject element : list) {
            element.lockThisObject();
        }
    }

    /**
     *
     * @return
     */
    public boolean isThisObjectLocked() {
        return this.locked;
    }

    /**
     * Throws UnsupportedOperationException, if this object is locked.
     */
    public void throwUnsupportedOperationExceptionIfThisObjectIsLocked() {
        if (this.isThisObjectLocked()) {
            throw new UnsupportedOperationException();
        }
    }

    /**
     * Throws UnsupportedOperationException, if this object is not valid.
     */
    public void throwUnsupportedOperationExceptionIfThisObjectIsNotValid() {
        if (!this.isThisObjectValid()) {
            throw new UnsupportedOperationException();
        }
    }

    /**
     *
     * @return
     */
    public int getRandomFrequency() {
        if (PseudoRandomGenerator.getInstance().getBoolean()) {
            return PseudoRandomGenerator.getInstance().getInt(1, 16);
        } else {
            return 0;
        }
    }

    /**
     *
     * @return
     */
    public int getNotNullRandomFrequency() {
        return PseudoRandomGenerator.getInstance().getInt(40, 100);
    }

    /**
     *
     * @return
     */
    public int getRandomProbability() {
        if (PseudoRandomGenerator.getInstance().getBoolean()) {
            return PseudoRandomGenerator.getInstance().getInt(0, 100);
        } else {
            return 0;
        }
    }

    /**
     *
     * @param probability
     * @return
     */
    public boolean isProbabilityValid(int probability) {
        return (probability >= 0) && (probability <= 100);
    }

    /**
     *
     * @return
     */
    public abstract boolean isThisObjectValid();

    /**
     * Sets default values.
     */
    public abstract void setDefaultValues();

    /**
     * Sets random values.
     */
    public abstract void setRandomValues();

    /**
     *
     * @return copy of the object
     */
    public abstract Object getCopy();

    /**
     *
     * @return
     */
    public abstract List getListOfLockableObjects();
}
