package cls.logic.game;

import cls.ColorLinesSequelRuntimeException;
import cls.logic.game.board.Field;
import java.util.ArrayList;
import okay.collections.Stack;

/**
 *
 * @author rober
 */
public class PathFinder {

    private final Stack<Field> pathFromStartToEndStack = new Stack<>();
    private final ArrayList<Field> visitedFields = new ArrayList<>();
    private final Stack<ParentNode<Field>> nodesBeforeExamined = new Stack<>();
    private final Stack<ParentNode<Field>> nodesAfterExamined = new Stack<>();
    private final ParentNode<Field> startNode;
    private ParentNode<Field> endNode;
    private boolean fieldToFound = false;
    private boolean fieldToNotFound = false;
    private final Field fieldFrom;
    private final Field fieldTo;
    private boolean pathWasSearched = false;
    private static final Direction[] directions = new Direction[]{Direction.TOP, Direction.RIGHT, Direction.BOTTOM, Direction.LEFT};

    /**
     *
     * @param fieldFrom
     * @param fieldTo
     */
    public PathFinder(Field fieldFrom, Field fieldTo) {
        startNode = new ParentNode<>(null, fieldFrom);
        this.fieldFrom = fieldFrom;
        this.fieldTo = fieldTo;

        nodesBeforeExamined.push(startNode);

        visitedFields.add(fieldFrom);

    }

    /**
     *
     * @return
     */
    public Stack<Field> findPath() {
        if (pathWasSearched) {
            throw new ColorLinesSequelRuntimeException("This method can be used only once.");
        }
        pathWasSearched = true;

        while (!this.fieldToFound && !this.fieldToNotFound) {
            moveAllNodesFromBeforeToAfterExamindes();
            examineAllNodesInNodesAfterExamined();
        }

        if (fieldToFound) {
            fillPathFromStartToEndStack();
        }
        return fieldToFound ? this.pathFromStartToEndStack : null;
    }

    private void moveAllNodesFromBeforeToAfterExamindes() {
        nodesAfterExamined.clear();
        if (this.nodesBeforeExamined.isEmpty()) {
            fieldToNotFound = true;
        }
        for (ParentNode<Field> node : nodesBeforeExamined) {
            nodesAfterExamined.push(node);
        }
    }

    private void examineAllNodesInNodesAfterExamined() {
        for (ParentNode<Field> node : nodesAfterExamined) {
            examineNode(node);
        }
    }

    private void examineNode(ParentNode<Field> node) {
        final Field fieldToExamin = node.getValue();
        if (fieldToExamin.equals(this.fieldTo)) {
            this.endNode = node;
            this.fieldToFound = true;

            return;
        }
        for (Direction direction : directions) {
            if (!fieldToExamin.getWalls().isWallAtDirection(direction)) {

                Field fieldBeforeExaming = fieldToExamin.getFieldAtDirection(direction);

                if (fieldBeforeExaming == null || this.visitedFields.contains(fieldBeforeExaming)) {
                    continue;
                }

                boolean addFieldWithWallsToVisitedFields = true;

                if (fieldBeforeExaming.getWalls().hasAWall()) {
                    addFieldWithWallsToVisitedFields = false;
                }

                if (addFieldWithWallsToVisitedFields) {
                    this.visitedFields.add(fieldBeforeExaming);
                }

                if (fieldBeforeExaming.isEmpty() && !fieldBeforeExaming.isLocked() && !fieldBeforeExaming.getWalls().isWallAtDirection(direction.overTurn())) {
                    this.nodesBeforeExamined.push(node.createChild(fieldBeforeExaming));
                }

            }
        }
    }

    private void fillPathFromStartToEndStack() {

        if (this.endNode != null) {
            ParentNode<Field> tempNode = endNode;
            while (tempNode.getParent() != null) {
                this.pathFromStartToEndStack.push(tempNode.getValue());
                tempNode = tempNode.getParent();
            }
        }
    }
}
