package cls.logic.game;

import cls.logic.game.board.Ball;
import cls.logic.game.board.Rollable;
import java.util.List;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class ResultCounter {

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private ResultCounter() {
        //Not meant to be instantiated.
    }

    /**
     * Counts score
     *
     * @param rollable
     * @param minRollableCountCount
     * @return
     */
    public static int countScore(List<Rollable> rollable, int minRollableCountCount) {
        int ballsCount = 0;
        int multiplier = 1;
        for (Rollable element : rollable) {
            if (!element.isBall()) {
                return 0;
            }
            Ball ball = (Ball) element;
            ++ballsCount;
            multiplier = multiplier * ball.getValue();
            if (multiplier == 0) {
                return 0;
            }
        }
        if (ballsCount < minRollableCountCount) {
            throw new cls.ColorLinesSequelRuntimeException("Balls were exploded, but their count (" + ballsCount + ") was smaller than the minimum (" + minRollableCountCount + ").");
        }
        int ballsOverMinimum = ballsCount - minRollableCountCount;
        switch (ballsOverMinimum) {
            case 0:
                return 10 * multiplier;
            case 1:
                return 12 * multiplier;
            case 2:
                return 18 * multiplier;
            case 3:
                return 28 * multiplier;
            case 4:
                return 42 * multiplier;
            default:
                return (42 + ((ballsOverMinimum - 4) * 5)) * multiplier;
        }
    }

}
