package cls.logic.game.board;

import cls.ColorLinesSequelRuntimeException;
import okay.json.JsonObject;

/**
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 */
public class Ball implements Rollable {

    private int colour;

    private int value;

    private final boolean unmoveable;
    private final boolean unbreakable;

    /**
     *
     * @param colour
     * @param value
     * @param unmoveable
     * @param unbreakable
     */
    public Ball(int colour, int value, boolean unmoveable, boolean unbreakable) {
        if (!isColourValid(colour)) {
            throw new ColorLinesSequelRuntimeException("Colour must be between 0 and 16.");
        }
        this.colour = colour;
        if (!isValueValid(value)) {
            throw new ColorLinesSequelRuntimeException("Value is out of range <-2;2>.");
        }
        this.value = value;
        this.unmoveable = unmoveable;
        this.unbreakable = unbreakable;
    }

    private boolean isColourValid(int colour) {
        return (colour >= 0) && (colour <= 16);
    }

    private boolean isValueValid(int value) {
        return (value >= -2) && (value <= 2);
    }

    /**
     *
     * @return
     */
    public int getColour() {
        return this.colour;
    }

    /**
     *
     * @return
     */
    public int getValue() {
        return this.value;
    }

    /**
     *
     * @return
     */
    public boolean isColorful() {
        return colour != 0;
    }

    /**
     *
     * @return
     */
    public boolean isRainbow() {
        return colour == 0;
    }

    /**
     *
     * @return
     */
    public boolean isUnmoveable() {
        return this.unmoveable;
    }

    /**
     *
     * @return
     */
    public boolean isUnbreakable() {
        return this.unbreakable;
    }

    @Override
    public String toString() {
        return colour
                + " "
                + value
                + " "
                + (unmoveable ? 1 : 0)
                + " "
                + (unbreakable ? 1 : 0);
    }

    @Override
    public boolean isBall() {
        return true;
    }

    @Override
    public boolean isBomb() {
        return false;
    }

    /**
     * Converts ball to json object
     *
     * @return
     */
    public JsonObject toJsonObject() {
        JsonObject jsonObject = new JsonObject();
        jsonObject.addString("colour", colour == 0 ? "RAINBOW" : okay.simplicity.EnumColour.convertNumberToEnumColour(colour).toString());
        jsonObject.addInt("value", value);
        jsonObject.addBoolean("unmoveable", this.unmoveable);
        jsonObject.addBoolean("unbreakable", this.unbreakable);
        return jsonObject;
    }
}
