package cls.services;

import cls.logic.Authentication;
import cls.logic.CreateNewPlayerResult;
import cls.logic.Logic;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class AuthenticationService {

    private final Logic logic;
    private final Authentication authentication;

    public AuthenticationService(Logic logic) {
        this.logic = logic;
        this.authentication = logic.getAuthentication();
    }

    /**
     * This method is used to create new player.
     *
     * @param nick <ul><li>From 1 to 16 characters.</li><li>Only a-z, A-Z and
     * 0-1 characters</li><li>This nick cannot be already used.</li></ul>
     * @param password <ul><li>From 0 to 16 characters.</li><li>Only a-z, A-Z
     * and 0-1 characters</li><li>For empty password use "".</li></ul>
     * @return True for successful login. False for failed login.
     */
    public CreateNewPlayerResult createNewPlayer(String nick, String password, String confirmPassword, int languageId, int skin, int zoom, int timeZoneId) {//NOSONAR
        return authentication.createNewPlayer(nick, password, confirmPassword, languageId, skin, zoom, timeZoneId);

    }

    public boolean logIn(String nick, String password, boolean keepingLogged) {
        return this.authentication.logIn(nick, password, keepingLogged);
    }

    /**
     * Log out current player. No player will be automatically logged in.
     */
    public void logOut() {
        authentication.logOut();
    }

    /**
     *
     * @return * If a player is logged in, returns true, otherwise returns
     * false.
     */
    public boolean isLoggedIn() {
        return logic.getPlayer() != null;
    }
}
