package cls.view.windows;

import cls.logic.CreateNewPlayerResult;
import cls.ColorLinesSequel;
import javafx.event.ActionEvent;
import javafx.geometry.Pos;
import javafx.stage.Modality;
import okay.simplicity.Simplicity;
import okay.simplicity.boxes.AlertBox;
import okay.simplicity.window.controls.Button;
import okay.simplicity.boxes.MessageBox;
import okay.simplicity.window.controls.SLabel;
import okay.simplicity.window.controls.PasswordField;
import okay.simplicity.window.controls.TextField;
import okay.simplicity.window.CellLayout;

/**
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 */
public class CreatingNewPlayerWindow extends ColorLinesSequelAbstractWindow {

    private CellLayout cellLayout;

    private SLabel displayLanguageLabel;
    private LanguageComboBox languageComboBox;

    private SLabel nickSLabel;
    private TextField nickSTextField;

    private SLabel passwordSLabel;
    private PasswordField passwordSPasswordField;

    private SLabel confirmPasswordSLabel;
    private PasswordField confirmPasswordSPasswordField;

    private SLabel passwordIsNotMandatorySLabel;
    private SLabel emptyPasswordFieldForNoPasswordSLabel;
    private SLabel restrictionsSLabel;
    private SLabel lengthSLabel;

    private SLabel colourSkinSLabel;
    private ColourComboBox colourComboBox;

    private Button createNewPlayerButton;

    /**
     * Constructor
     */
    public CreatingNewPlayerWindow() {
    }

    /**
     *
     */
    @Override
    public void initAreaForUserInteraction() {
        initModality(Modality.APPLICATION_MODAL);
        this.setWidth(120 * Simplicity.getDpmm());
        this.setHeight((97.5 + (2 / 3)) * Simplicity.getDpmm());

        this.showOnlyTheCloseButton();
        cellLayout = new CellLayout(2, 10, "50% 50%");

        displayLanguageLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(211) + ":");
        languageComboBox = new LanguageComboBox();

        nickSLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(111) + ":");
        nickSTextField = new TextField();

        passwordSLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(112) + ":");
        passwordSPasswordField = new PasswordField();

        confirmPasswordSLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(214) + ":");
        confirmPasswordSPasswordField = new PasswordField();

        passwordIsNotMandatorySLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(221));
        emptyPasswordFieldForNoPasswordSLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(222));
        restrictionsSLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(223));
        lengthSLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(224));

        colourSkinSLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(231) + ":");
        colourComboBox = new ColourComboBox();

        createNewPlayerButton = new Button(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(133));

        this.createNewPlayerButton.setOnAction(this::handleCreateNewPlayerButtonAction);
        this.setWindowTitle(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(201));

        this.applicationArea.getChildren().addAll(this.cellLayout);

        this.applicationArea.setAlignment(Pos.BOTTOM_CENTER);

        cellLayout.addNodes(1, displayLanguageLabel, languageComboBox);
        cellLayout.addNodes(2, nickSLabel, nickSTextField);
        cellLayout.addNodes(3, passwordSLabel, passwordSPasswordField);
        cellLayout.addNodes(4, confirmPasswordSLabel, confirmPasswordSPasswordField);
        cellLayout.addNode(passwordIsNotMandatorySLabel, 1, 5, 2, 1);
        cellLayout.addNode(emptyPasswordFieldForNoPasswordSLabel, 1, 6, 2, 1);
        cellLayout.addNode(restrictionsSLabel, 1, 7, 2, 1);
        cellLayout.addNode(lengthSLabel, 1, 8, 2, 1);
        cellLayout.addNodes(9, colourSkinSLabel, colourComboBox);
        cellLayout.addNode(createNewPlayerButton, 2, 10);
    }

    private void handleCreateNewPlayerButtonAction(ActionEvent event) {
        String nick;
        String password;
        String confirmPassword;
        int languageID;
        int skin;

        languageID = languageComboBox.getSelectedLanguageId();

        nick = this.nickSTextField.getText().toString();
        password = this.passwordSPasswordField.getText().toString();
        confirmPassword = this.confirmPasswordSPasswordField.getText().toString();

        skin = colourComboBox.getColourNumber();

        CreateNewPlayerResult result = ColorLinesSequel.getServices().getAuthenticationService().createNewPlayer(nick, password, confirmPassword, languageID, skin, 100, 1);
        switch (result) {
            case PLAYER_CREATED: {
                MessageBox.showBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(1), cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(1002));
                this.close();
            }
            break;
            case NICK_ALREADY_EXISTS:
                AlertBox.showBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(2), cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(244));
                break;
            case TOO_LONG_NICK:
                AlertBox.showBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(2), cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(241));
                break;
            case NICK_IS_EMPTY:
                AlertBox.showBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(2), cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(243));
                break;
            case PASSWORD_AND_CONFIRM_PASSWORD_DO_NOT_MATCH:
                AlertBox.showBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(2), cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(245));
                break;
            case TOO_LONG_PASSWORD:
                AlertBox.showBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(2), cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(242));
                break;
            case NICK_OR_PASSWORD_CONTAINS_FORBIDDEN_CHARACTERS:
                AlertBox.showBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(2), cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(223));
                break;
            default:
                throw new IllegalStateException();
        }

    }
}
