package cls.view.windows.customcompositionwindow;

import javafx.scene.control.Tab;
import okay.json.JsonObject;
import okay.simplicity.window.CellLayout;
import okay.simplicity.window.controls.SLabel;
import okay.simplicity.window.controls.TextField;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class BallsOtherTab extends Tab {

    private SLabel colourfulBallFrequencyLabel;
    private SLabel rainbowBallFrequencyLabel;
    private SLabel unmoveableBallsProbabilityLabel;
    private SLabel unbreakableBallsProbabilityLabel;

    private TextField colourfulBallFrequencyTextField;
    private TextField rainbowBallFrequencyTextField;
    private TextField unmoveableBallsProbabilityTextField;
    private TextField unbreakableBallsProbabilityTextField;

    private final CellLayout cellLayout;

    /**
     *
     */
    public BallsOtherTab() {
        super(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20240));
        this.setClosable(false);
        cellLayout = initCellLayout();

        initNodes();

        fillCellLayout();
    }

    private void fillCellLayout() {
        cellLayout.addNode(colourfulBallFrequencyLabel, 2, 1);
        cellLayout.addNode(colourfulBallFrequencyTextField, 3, 1);
        cellLayout.addNode(rainbowBallFrequencyLabel, 2, 2);
        cellLayout.addNode(rainbowBallFrequencyTextField, 3, 2);
        cellLayout.addNode(unmoveableBallsProbabilityLabel, 2, 3);
        cellLayout.addNode(unmoveableBallsProbabilityTextField, 3, 3);
        cellLayout.addNode(unbreakableBallsProbabilityLabel, 2, 4);
        cellLayout.addNode(unbreakableBallsProbabilityTextField, 3, 4);

        setContent(cellLayout);
    }

    private CellLayout initCellLayout() {
        CellLayout cellLayout = new CellLayout(3, 5, "10% 70% 20%");
        return cellLayout;
    }

    private void initNodes() {
        initLabels();
        initTextFields();
    }

    private void initLabels() {
        colourfulBallFrequencyLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20241));
        rainbowBallFrequencyLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20242));
        unmoveableBallsProbabilityLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20243));
        unbreakableBallsProbabilityLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20244));
    }

    private void initTextFields() {
        colourfulBallFrequencyTextField = new TextField();
        rainbowBallFrequencyTextField = new TextField();
        unmoveableBallsProbabilityTextField = new TextField();
        unbreakableBallsProbabilityTextField = new TextField();
    }

    /**
     *
     * @return
     */
    public int getColourfulBallFrequency() {
        return Integer.parseInt(colourfulBallFrequencyTextField.getText());
    }

    /**
     *
     * @param value
     */
    public void setColourfulBallFrequency(int value) {
        this.colourfulBallFrequencyTextField.setText(String.valueOf(value));
    }

    /**
     *
     * @return
     */
    public int getRainbowBallFrequency() {
        return Integer.parseInt(rainbowBallFrequencyTextField.getText());
    }

    /**
     *
     * @param value
     */
    public void setRainbowBallFrequency(int value) {
        this.rainbowBallFrequencyTextField.setText(String.valueOf(value));
    }

    /**
     *
     * @return
     */
    public int getUnmoveableBallsProbability() {
        return Integer.parseInt(unmoveableBallsProbabilityTextField.getText());
    }

    /**
     *
     * @param value
     */
    public void setUnmoveableBallsProbability(int value) {
        this.unmoveableBallsProbabilityTextField.setText(String.valueOf(value));
    }

    /**
     *
     * @return
     */
    public int getUnbreakableBallsProbability() {
        return Integer.parseInt(unbreakableBallsProbabilityTextField.getText());
    }

    /**
     *
     * @param value
     */
    public void setUnbreakableBallsProbability(int value) {
        this.unbreakableBallsProbabilityTextField.setText(String.valueOf(value));
    }

    /**
     *
     * @param jsonObject
     */
    public void updateByJsonObject(JsonObject jsonObject) {
        this.setColourfulBallFrequency(jsonObject.getInt("colorful ball frequency"));
        this.setRainbowBallFrequency(jsonObject.getInt("rainbow ball frequency"));
        this.setUnmoveableBallsProbability(jsonObject.getInt("unmoveable ball probability"));
        this.setUnbreakableBallsProbability(jsonObject.getInt("unbreakable ball probability"));
    }
}
