package cls.view.windows.customcompositionwindow;

import javafx.scene.control.Tab;
import okay.json.JsonArray;
import okay.json.JsonObject;
import okay.simplicity.window.controls.TabPane;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class BallsTab extends Tab {

    private final TabPane tabPane;
    private final ColourFrequenciesTab colourFrequenciesTab;
    private final ValuesFrequenciesTab valuesFrequenciesTab;
    private final BallsOtherTab ballsOtherTab;

    /**
     *
     */
    public BallsTab() {
        super(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20200));
        this.setClosable(false);
        this.colourFrequenciesTab = new ColourFrequenciesTab();
        this.valuesFrequenciesTab = new ValuesFrequenciesTab();
        this.ballsOtherTab = new BallsOtherTab();
        this.tabPane = new TabPane();
        tabPane.getTabs().addAll(colourFrequenciesTab, valuesFrequenciesTab, ballsOtherTab);
        this.setContent(tabPane);
    }

    /**
     *
     * @param jsonObject
     */
    public void updateByJsonObject(JsonObject jsonObject) {
        this.colourFrequenciesTab.updateByJsonObject(jsonObject);
        this.valuesFrequenciesTab.updateByJsonObject(jsonObject);
        this.ballsOtherTab.updateByJsonObject(jsonObject);
    }

    /**
     *
     * @return
     */
    public JsonObject toJsonObject() {
        JsonObject ballFactoryCompositionJsonObject = new JsonObject();
        JsonArray colourFrequencyJsonArray = this.colourFrequenciesTab.toJsonArray();

        ballFactoryCompositionJsonObject.addArray("colour frequency", colourFrequencyJsonArray);

        ballFactoryCompositionJsonObject.addInt("value minus two frequency", this.valuesFrequenciesTab.getMinusTwoFrequency());
        ballFactoryCompositionJsonObject.addInt("value minus one frequency", this.valuesFrequenciesTab.getMinusOneFrequency());
        ballFactoryCompositionJsonObject.addInt("value zero frequency", this.valuesFrequenciesTab.getZeroFrequency());
        ballFactoryCompositionJsonObject.addInt("value plus one frequency", this.valuesFrequenciesTab.getPlusOneFrequency());
        ballFactoryCompositionJsonObject.addInt("value plus two frequency", this.valuesFrequenciesTab.getPlusTwoFrequency());

        ballFactoryCompositionJsonObject.addInt("colorful ball frequency", this.ballsOtherTab.getColourfulBallFrequency());
        ballFactoryCompositionJsonObject.addInt("rainbow ball frequency", this.ballsOtherTab.getRainbowBallFrequency());
        ballFactoryCompositionJsonObject.addInt("unmoveable ball probability", this.ballsOtherTab.getUnmoveableBallsProbability());
        ballFactoryCompositionJsonObject.addInt("unbreakable ball probability", this.ballsOtherTab.getUnbreakableBallsProbability());

        return ballFactoryCompositionJsonObject;
    }
}
