package cls.view.windows.customcompositionwindow;

import cls.ColorLinesSequelRuntimeException;
import javafx.scene.control.Tab;
import javafx.scene.control.ToggleGroup;
import okay.json.JsonArray;
import okay.json.JsonObject;
import okay.simplicity.window.CellLayout;
import okay.simplicity.window.controls.Button;
import okay.simplicity.window.controls.RadioButton;
import okay.simplicity.window.controls.SLabel;
import okay.simplicity.window.controls.TextField;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class DetonatorTab extends Tab {

    private SLabel shapeToExplodeLabel;

    private SLabel minimumLenghtLabel;
    private SLabel minimumSizeLabel;

    private RadioButton lineRadioButton;
    private RadioButton blockRadioButton;
    private RadioButton squareRadioButton;
    private RadioButton ringRadioButton;
    private RadioButton customRadioButton;

    private TextField minimumLenghtTextField;
    private TextField minimumSizeTextField;

    private Button setCustomShapeToExplodeButton;

    private CellLayout cellLayout;

    private ToggleGroup toogleGroup;

    /**
     *
     */
    public DetonatorTab() {
        super(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20400));
        this.setClosable(false);
        cellLayout = initCellLayout();

        initNodes();

        fillCellLayout();
    }

    private void fillCellLayout() {
        cellLayout.addNodes(1, this.shapeToExplodeLabel);
        cellLayout.addNodes(2, this.lineRadioButton, this.minimumLenghtLabel, this.minimumLenghtTextField);
//        cellLayout.addNodes(3, this.blockRadioButton, this.minimumSizeLabel, this.minimumSizeTextField);
        cellLayout.addNodes(3, this.squareRadioButton);
        cellLayout.addNodes(4, this.ringRadioButton);
//        cellLayout.addNode(this.customRadioButton, 1, 6);
//        cellLayout.addNode(this.setCustomShapeToExplodeButton, 2, 6, 2, 1);

        setContent(cellLayout);
    }

    private CellLayout initCellLayout() {
        CellLayout cellLayout = new CellLayout(3, 6, "50% 25% 25%");
        return cellLayout;
    }

    private void initNodes() {
        initRadioButtons();
        initLabels();
        initTextFields();
        initButtons();
    }

    private void initRadioButtons() {
        lineRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20402));
        blockRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20403));
        squareRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20404));
        ringRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20405));
        customRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20406));

        toogleGroup = new ToggleGroup();

        lineRadioButton.setToggleGroup(toogleGroup);
        blockRadioButton.setToggleGroup(toogleGroup);
        squareRadioButton.setToggleGroup(toogleGroup);
        ringRadioButton.setToggleGroup(toogleGroup);
        customRadioButton.setToggleGroup(toogleGroup);
    }

    private void initButtons() {
        this.setCustomShapeToExplodeButton = new Button(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20421));
    }

    private void initLabels() {
        shapeToExplodeLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20401));

        minimumLenghtLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20411));
        minimumSizeLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(20412));
    }

    private void initTextFields() {
        minimumLenghtTextField = new TextField();
        minimumSizeTextField = new TextField();
    }

    /**
     *
     * @param jsonObject
     */
    public void updateByJsonObject(JsonObject jsonObject) {
        String explodingShapeType = jsonObject.getString("exploding shape type");
        switch (explodingShapeType) {
            case "LINE":
                this.lineRadioButton.setSelected(true);
                break;
            case "BLOCK":
                this.blockRadioButton.setSelected(true);
                break;
            case "RING":
                this.ringRadioButton.setSelected(true);
                break;
            case "SQUARE":
                this.squareRadioButton.setSelected(true);
                break;
            case "CUSTOM":
                this.customRadioButton.setSelected(true);
                break;
        }
        this.minimumLenghtTextField.setText(String.valueOf(jsonObject.getInt("minimum line lenght")));
        this.minimumSizeTextField.setText(String.valueOf(jsonObject.getInt("minimum block size")));
    }

    /**
     *
     * @return
     */
    public JsonObject toJsonObject() {
        JsonObject ballDetonatorCompositionJsonObject = new JsonObject();

        String explodingShapeType;

        if (lineRadioButton.isSelected()) {
            explodingShapeType = "LINE";
        } else if (blockRadioButton.isSelected()) {
            explodingShapeType = "BLOCK";
        } else if (ringRadioButton.isSelected()) {
            explodingShapeType = "RING";
        } else if (squareRadioButton.isSelected()) {
            explodingShapeType = "SQUARE";
        } else if (customRadioButton.isSelected()) {
            explodingShapeType = "CUSTOM";
        } else {
            throw new ColorLinesSequelRuntimeException("Fatal error.");
        }

        ballDetonatorCompositionJsonObject.addString("exploding shape type", explodingShapeType);
        ballDetonatorCompositionJsonObject.addInt("minimum line lenght", Integer.parseInt(this.minimumLenghtTextField.getText()));
        ballDetonatorCompositionJsonObject.addInt("minimum block size", Integer.parseInt(this.minimumSizeTextField.getText()));
        if (!("CUSTOM".equals(explodingShapeType))) {
            ballDetonatorCompositionJsonObject.addNull("custom exploding shape");
        } else {
            JsonObject shapeJsonObject = new JsonObject();
            shapeJsonObject.addInt("height", 3);
            shapeJsonObject.addInt("width", 3);

            shapeJsonObject.addArray("hole list", new JsonArray());

            ballDetonatorCompositionJsonObject.addObject("custom exploding shape", shapeJsonObject);
        }

        return ballDetonatorCompositionJsonObject;
    }
}
