package cls.view.windows.playersettingswindow;

import cls.ColorLinesSequel;
import cls.view.windows.ColourComboBox;
import cls.view.windows.LanguageComboBox;
import javafx.event.Event;
import javafx.scene.control.Tab;
import okay.simplicity.EnumColour;
import okay.simplicity.window.CellLayout;
import okay.simplicity.window.controls.Button;
import okay.simplicity.window.controls.SLabel;
import okay.simplicity.window.controls.TextField;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class BeforeSigningTab extends Tab {

    private SLabel displayLanguageLabel;
    private SLabel colourSkinLabel;
    private SLabel zoomLabel;

    private LanguageComboBox languageComboBox;
    private ColourComboBox colourComboBox;

    private TextField zoomTextField;

    private Button saveBeforeSigningChangesButton;

    private final CellLayout cellLayout;

    /**
     *
     */
    public BeforeSigningTab() {
        super(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10901));
        this.setClosable(false);
        cellLayout = initCellLayout();

        initNodes();

        fillCellLayout();
    }

    private void fillCellLayout() {
        cellLayout.addNodes(1, this.displayLanguageLabel, this.languageComboBox);
        cellLayout.addNodes(2, this.colourSkinLabel, this.colourComboBox);
        cellLayout.addNodes(3, this.zoomLabel, this.zoomTextField);
        cellLayout.addNode(this.saveBeforeSigningChangesButton, 2, 5);

        setContent(cellLayout);
    }

    private CellLayout initCellLayout() {
        CellLayout cellLayout = new CellLayout(2, 5, "50% 50%");
        return cellLayout;
    }

    private void initNodes() {
        initLabels();
        initComboBoxes();
        initTextFields();
        initButtons();
    }

    private void initLabels() {
        displayLanguageLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(211));
        colourSkinLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(231));
        zoomLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10102));
    }

    private void initComboBoxes() {
        this.languageComboBox = new LanguageComboBox();
        this.languageComboBox.setValue(ColorLinesSequel.getServices().getLanguageService().convertLanguageIdToLanguageName(ColorLinesSequel.getServices().getBeforeLoginService().getLanguageId()));
        this.colourComboBox = new ColourComboBox();
        this.colourComboBox.setColourNumber(ColorLinesSequel.getServices().getBeforeLoginService().getColourSkin().getNumber());
    }

    private void initTextFields() {
        this.zoomTextField = new TextField(String.valueOf(ColorLinesSequel.getServices().getBeforeLoginService().getZoom()));
    }

    private void initButtons() {
        this.saveBeforeSigningChangesButton = new Button(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10991));
        this.saveBeforeSigningChangesButton.setOnAction(this::handleSaveLookChangesButtonAction);
    }

    private void handleSaveLookChangesButtonAction(Event e) {
        int currentBeforeSigningLanguageId = this.languageComboBox.getSelectedLanguageId();
        ColorLinesSequel.getServices().getBeforeLoginService().setLanguageId(currentBeforeSigningLanguageId);

        int currentBeforeSigningColourNumber = this.colourComboBox.getColourNumber();
        ColorLinesSequel.getServices().getBeforeLoginService().setColourSkin(EnumColour.convertNumberToEnumColour(currentBeforeSigningColourNumber));

        int currentBeforeSigningZoom = Integer.parseInt(this.zoomTextField.getText());
        ColorLinesSequel.getServices().getBeforeLoginService().setZoom(currentBeforeSigningZoom);
    }
}
