package cls.view.windows.playersettingswindow;

import cls.ColorLinesSequel;
import cls.view.windows.ColourComboBox;
import cls.view.windows.LanguageComboBox;
import javafx.event.Event;
import javafx.scene.control.Tab;
import okay.simplicity.window.CellLayout;
import okay.simplicity.window.controls.RadioButton;
import okay.simplicity.window.controls.SCheckBox;
import okay.simplicity.window.controls.SLabel;
import okay.simplicity.window.controls.TextField;
import javafx.scene.control.ToggleGroup;
import okay.simplicity.EnumColour;
import okay.simplicity.window.controls.Button;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class LookTab extends Tab {

    private SLabel displayLanguageLabel;
    private SLabel colourSkinLabel;
    private SLabel zoomLabel;
    private SLabel ballLightingLabel;
    private SLabel ballMoveEffectLabel;

    private LanguageComboBox languageComboBox;
    private ColourComboBox colourComboBox;

    private TextField zoomTextField;

    private RadioButton noneRadioButton;
    private RadioButton aboveRadioButton;
    private RadioButton aheadRadioButton;

    private ToggleGroup ballLightingToogleGroup;

    private SCheckBox showLineAroundBallCheckBox;
    private SCheckBox showWhereABallCanBeMovedCheckBox;
    private SCheckBox highlightFieldAfterBallExplodedCheckBox;

    private RadioButton noExtraEffectRadioButton;
    private RadioButton arrowRadioButton;
    private RadioButton highlightFieldsRadioButton;

    private ToggleGroup ballMoveEffectToogleGroup;

    private Button loadDefaultButton;
    private Button saveLookChanges;

    private final int languageId;
    private final int colourNumber;
    private final int zoom;
    private final String ballLightingName;
    private final boolean linesAround;
    private final boolean showBallMoves;
    private final boolean highlightFieldBecauseOfExplosion;
    private final String ballMoveEffectName;

    private final CellLayout cellLayout;

    /**
     *
     */
    public LookTab() {
        super(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10101));
        this.setClosable(false);

        this.languageId = ColorLinesSequel.getServices().getLookService().getLanguageId();
        this.colourNumber = ColorLinesSequel.getServices().getLookService().getColourSkin().getNumber();
        this.zoom = (int) ColorLinesSequel.getServices().getLookService().getZoom();
        this.ballLightingName = ColorLinesSequel.getServices().getPlayerService().getBallLightingName();
        this.linesAround = ColorLinesSequel.getServices().getPlayerService().getLinesAround();
        this.showBallMoves = ColorLinesSequel.getServices().getPlayerService().getShowWhereABallCanBeMoved();
        this.highlightFieldBecauseOfExplosion = ColorLinesSequel.getServices().getPlayerService().getHighlightFieldAfterBallExploded();
        this.ballMoveEffectName = ColorLinesSequel.getServices().getPlayerService().getBallMoveEffectName();

        cellLayout = initCellLayout();

        initNodes();

        fillCellLayout();
    }

    private void fillCellLayout() {
        cellLayout.addNodes(1, this.displayLanguageLabel, this.languageComboBox);
        cellLayout.addNodes(2, this.colourSkinLabel, this.colourComboBox);
        cellLayout.addNodes(3, this.zoomLabel, this.zoomTextField);
        cellLayout.addNodes(4, this.ballLightingLabel, this.noneRadioButton);
        cellLayout.addNode(this.aboveRadioButton, 2, 5);
        cellLayout.addNode(this.aheadRadioButton, 2, 6);
        cellLayout.addNode(this.showLineAroundBallCheckBox, 1, 7, 2, 1);
//        cellLayout.addNode(this.showWhereABallCanBeMovedCheckBox, 1, 8, 2, 1);
//        cellLayout.addNode(this.highlightFieldAfterBallExplodedCheckBox, 1, 9, 2, 1);
        cellLayout.addNodes(8, this.ballMoveEffectLabel, this.noExtraEffectRadioButton);
//        cellLayout.addNode(this.arrowRadioButton, 2, 11);
        cellLayout.addNode(this.highlightFieldsRadioButton, 2, 9);
        cellLayout.addNodes(10, this.loadDefaultButton, this.saveLookChanges);

        setContent(cellLayout);
    }

    private CellLayout initCellLayout() {
        CellLayout cellLayout = new CellLayout(2, 13, "50% 50%");
        return cellLayout;
    }

    private void initNodes() {
        initLabels();
        initComboBoxes();
        initTextFields();
        initRadioButtons();
        initCheckBoxes();
        initRadioButtons();
        initButtons();
    }

    private void initLabels() {
        displayLanguageLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(211));
        colourSkinLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(231));
        zoomLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10102));
        ballLightingLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10113));
        ballMoveEffectLabel = new SLabel(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10124));
    }

    private void initComboBoxes() {
        this.languageComboBox = new LanguageComboBox();
        this.colourComboBox = new ColourComboBox();
        this.colourComboBox.setColourNumber(ColorLinesSequel.getServices().getLookService().getColourSkin().getNumber());
    }

    private void initTextFields() {
        this.zoomTextField = new TextField(String.valueOf(this.zoom));
    }

    private void initRadioButtons() {
        noneRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10114));
        this.noneRadioButton.setId("none");
        if ("none".equals(this.ballLightingName)) {
            noneRadioButton.setSelected(true);
        }

        aboveRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10115));
        this.aboveRadioButton.setId("above");
        if ("above".equals(this.ballLightingName)) {
            aboveRadioButton.setSelected(true);
        }

        aheadRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10116));
        this.aheadRadioButton.setId("ahead");
        if ("ahead".equals(this.ballLightingName)) {
            aheadRadioButton.setSelected(true);
        }

        ballLightingToogleGroup = new ToggleGroup();

        noneRadioButton.setToggleGroup(ballLightingToogleGroup);
        aboveRadioButton.setToggleGroup(ballLightingToogleGroup);
        aheadRadioButton.setToggleGroup(ballLightingToogleGroup);

        noExtraEffectRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10125));
        this.noExtraEffectRadioButton.setId("no");
        if ("no".equals(this.ballMoveEffectName)) {
            noExtraEffectRadioButton.setSelected(true);
        }

        arrowRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10126));
        this.arrowRadioButton.setId("arrow");
        if ("arrow".equals(this.ballMoveEffectName)) {
            arrowRadioButton.setSelected(true);
        }

        highlightFieldsRadioButton = new RadioButton(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10127));
        this.highlightFieldsRadioButton.setId("highlight");
        if ("highlight".equals(this.ballMoveEffectName)) {
            highlightFieldsRadioButton.setSelected(true);
        }

        ballMoveEffectToogleGroup = new ToggleGroup();

        noExtraEffectRadioButton.setToggleGroup(ballMoveEffectToogleGroup);
        arrowRadioButton.setToggleGroup(ballMoveEffectToogleGroup);
        highlightFieldsRadioButton.setToggleGroup(ballMoveEffectToogleGroup);

    }

    private void initCheckBoxes() {
        showLineAroundBallCheckBox = new SCheckBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10121));
        showLineAroundBallCheckBox.setSelected(this.linesAround);

        showWhereABallCanBeMovedCheckBox = new SCheckBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10122));
        showWhereABallCanBeMovedCheckBox.setSelected(this.showBallMoves);

        highlightFieldAfterBallExplodedCheckBox = new SCheckBox(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10123));
        highlightFieldAfterBallExplodedCheckBox.setSelected(this.highlightFieldBecauseOfExplosion);
    }

    private void initButtons() {
        this.loadDefaultButton = new Button(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10140));
        this.loadDefaultButton.setOnAction(this::handleLoadDefaultButtonAction);
        this.saveLookChanges = new Button(cls.ColorLinesSequel.getServices().getLanguageService().getTextConstant(10141));
        this.saveLookChanges.setOnAction(this::handleSaveLookChangesButtonAction);
    }

    private void handleLoadDefaultButtonAction(Event e) {
        this.languageComboBox.setDefault();
    }

    private void handleSaveLookChangesButtonAction(Event e) {
        int currentLanguageId = this.languageComboBox.getSelectedLanguageId();
        ColorLinesSequel.getServices().getLookService().setLanguageId(currentLanguageId);

        int currentColourNumber = this.colourComboBox.getColourNumber();
        ColorLinesSequel.getServices().getLookService().setColourSkin(EnumColour.convertNumberToEnumColour(currentColourNumber));

        int currentZoom = Integer.parseInt(this.zoomTextField.getText());
        ColorLinesSequel.getServices().getLookService().setZoom(currentZoom);

        String currentBallLightingName = ((RadioButton) this.ballLightingToogleGroup.getSelectedToggle()).getId();
        ColorLinesSequel.getServices().getPlayerService().setBallLightingName(currentBallLightingName);

        boolean currentShowLineAroundBall = this.showLineAroundBallCheckBox.isSelected();
        ColorLinesSequel.getServices().getPlayerService().setLinesAround(currentShowLineAroundBall);

        boolean currentshowWhereABallCanBeMoved = this.showWhereABallCanBeMovedCheckBox.isSelected();
        ColorLinesSequel.getServices().getPlayerService().setShowWhereABallCanBeMoved(currentshowWhereABallCanBeMoved);

        boolean currentHighlightFieldAfterBallExploded = this.highlightFieldAfterBallExplodedCheckBox.isSelected();
        ColorLinesSequel.getServices().getPlayerService().setShowHighlightFieldAfterBallExploded(currentHighlightFieldAfterBallExploded);

        String currentBallMoveEffectName = ((RadioButton) this.ballMoveEffectToogleGroup.getSelectedToggle()).getId();
        ColorLinesSequel.getServices().getPlayerService().setBallMoveEffectName(currentBallMoveEffectName);
    }
}
